@@ -6,30 +6,23 @@
 # @Filename: actor.py
 # @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
 
-import asyncio
 import json
 import re
-import time
 import uuid
-from contextlib import suppress
+
+import aio_pika as apika
+import click
 
 from .base import BaseActor
 from .client import AMQPClient
-from .command import Command
+from .command import Command, TimedCommandList
 from .exceptions import CommandError
-from .model import ModelSet
-from .parser import ClickParser
+from .parser import ClickParser, CluCommand
 from .protocol import TCPStreamServer
 from .tools import log_reply
 
 
-try:
-    import aio_pika as apika
-except ImportError:
-    apika = None
-
-
-__all__ = ['AMQPActor', 'JSONActor', 'TimerCommand', 'TimerCommandList']
+__all__ = ['AMQPActor', 'JSONActor']
 
 
 class AMQPActor(AMQPClient, ClickParser, BaseActor):
@@ -41,32 +34,31 @@ class AMQPActor(AMQPClient, ClickParser, BaseActor):
     internals and protocols are different the entry points and behaviour for
     both classes should be almost identical.
 
-    See the documentation for `.AMQPClient` for parameter information.
+    See the documentation for `.AMQPActor` and `.AMQPClient` for additional
+    parameter information.
+
+    Parameters
+    ----------
+    schema : str
+        The path to the datamodel schema for the actor, in JSON Schema format.
+        If the schema is provided all replies will be validated against it.
+        An invalid reply will fail and not be emitted. The schema can also be
+        set when subclassing by setting the class ``schema`` attribute.
+
 
     """
 
-    def __init__(self, *args, model_path=None, model_names=None, **kwargs):
+    def __init__(self, *args, schema=None, **kwargs):
 
         AMQPClient.__init__(self, *args, **kwargs)
 
         self.commands_queue = None
 
-        if model_path:
+        self.timed_commands = TimedCommandList(self)
 
-            model_names = model_names or []
-
-            if self.name not in model_names:
-                model_names.append(self.name)
-
-            self.models = ModelSet(model_path, model_names=model_names,
-                                   raise_exception=False, log=self.log)
-
-        else:
-
-            self.log.warning('no models loaded.')
-            self.models = None
-
-        self.timer_commands = TimerCommandList(self)
+        # Not calling BaseClient.__init__() here because we already called
+        # AMQPClient.__init__.
+        self.validate_schema(schema)
 
     async def start(self, **kwargs):
         """Starts the connection to the AMQP broker."""
@@ -82,14 +74,14 @@ class AMQPActor(AMQPClient, ClickParser, BaseActor):
         self.log.info(f'commands queue {self.commands_queue.name!r} '
                       f'bound to {self.connection.connection.url!s}')
 
-        self.timer_commands.start()
+        self.timed_commands.start()
 
         return self
 
     async def new_command(self, message):
         """Handles a new command received by the actor."""
 
-        with message.process():
+        async with message.process():
 
             headers = message.info()['headers']
             command_body = json.loads(message.body.decode())
@@ -105,31 +97,15 @@ class AMQPActor(AMQPClient, ClickParser, BaseActor):
                               actor=self, loop=self.loop)
             command.actor = self  # Assign the actor
         except CommandError as ee:
-            await self.write('f', {'text': f'Could not parse the '
-                                           f'following as a command: {ee!r}'})
+            await self.write('f', {'error': 'Could not parse the '
+                                            'following as a command: '
+                                            f'{command_string!r}. {ee!r}'})
             return
 
         return self.parse_command(command)
 
-    async def handle_reply(self, message):
-        """Handles a reply received by the message and updates the models.
-
-        Parameters
-        ----------
-        message : aio_pika.IncomingMessage
-            The message received.
-
-        """
-
-        reply = await AMQPClient.handle_reply(self, message)
-
-        if self.models and reply.sender in self.models:
-            self.models[reply.sender].update_model(reply.body)
-
-        return
-
     async def write(self, message_code='i', message=None, command=None,
-                    broadcast=False, **kwargs):
+                    broadcast=False, no_validate=False, **kwargs):
         """Writes a message to user(s).
 
         Parameters
@@ -145,6 +121,10 @@ class AMQPActor(AMQPClient, ClickParser, BaseActor):
         broadcast : bool
             Whether to broadcast the message to all the users or only to the
             commander.
+        no_validate : bool
+            Do not validate the reply against the actor schema. This is
+            ignored if the actor was not started with knowledge of its own
+            schema.
         kwargs
             Keyword arguments that will be added to the message.
 
@@ -156,6 +136,11 @@ class AMQPActor(AMQPClient, ClickParser, BaseActor):
 
         message.update(kwargs)
 
+        if not no_validate and self.schema is not None:
+            result, err = self.schema.update_model(message)
+            if result is False:
+                message = {'error': f'Failed validating the reply: {err}'}
+
         message_json = json.dumps(message)
 
         if command is None:
@@ -210,13 +195,16 @@ class JSONActor(ClickParser, BaseActor):
 
     """
 
-    def __init__(self, name, host, port, *args, **kwargs):
+    def __init__(self, name, host=None, port=None, *args, **kwargs):
 
         super().__init__(name, *args, **kwargs)
 
-        self.host = host
+        self.host = host or 'localhost'
         self.port = port
 
+        if self.port is None:
+            raise ValueError('port needs to be specified.')
+
         #: Mapping of commander_id to transport
         self.transports = dict()
 
@@ -225,18 +213,29 @@ class JSONActor(ClickParser, BaseActor):
                                       connection_callback=self.new_user,
                                       data_received_callback=self.new_command)
 
-        self.timer_commands = TimerCommandList(self)
+        self.timed_commands = TimedCommandList(self)
+
+        # Add the multiline command
+        self.parser.add_command(multiline)
 
     async def start(self):
         """Starts the TCP server."""
 
-        await self.server.start_server()
+        await self.server.start()
         self.log.info(f'running TCP server on {self.host}:{self.port}')
 
-        self.timer_commands.start()
+        self.timed_commands.start()
 
         return self
 
+    async def stop(self):
+        """Stops the client connection and running tasks."""
+
+        if self.server.is_serving():
+            self.server.stop()
+
+        await self.timed_commands.stop()
+
     async def run_forever(self):
         """Runs the actor forever, keeping the loop alive."""
 
@@ -252,6 +251,7 @@ class JSONActor(ClickParser, BaseActor):
 
         user_id = str(uuid.uuid4())
         transport.user_id = user_id
+        transport.multiline = False
         self.transports[user_id] = transport
 
         sock = transport.get_extra_info('socket')
@@ -294,13 +294,13 @@ class JSONActor(ClickParser, BaseActor):
 
         return self.parse_command(command)
 
-    def send_command(self):
+    def send_command(self, *args, **kwargs):
         """Not implemented for `.JSONActor`."""
 
         raise NotImplementedError('JSONActor cannot send commands to other actors.')
 
     def write(self, message_code='i', message=None, command=None,
-              broadcast=False, beautify=True, **kwargs):
+              broadcast=False, no_validate=False, **kwargs):
         """Writes a message to user(s) as a JSON.
 
         A ``header`` keyword with the ``commander_id`` (i.e., the user id of
@@ -329,6 +329,11 @@ class JSONActor(ClickParser, BaseActor):
                 }
             }
 
+        Although the messsage is displayed here in multiple lines, it is
+        written as a single line to the TCP clients to facilitate parsing.
+        For a multiline output, which is more human-readable, use the
+        ``multiline`` command.
+
         Parameters
         ----------
         message_code : str
@@ -342,17 +347,32 @@ class JSONActor(ClickParser, BaseActor):
         broadcast : bool
             Whether to broadcast the message to all the users or only to the
             commander.
-        beautify : bool
-            Whether to format the JSON to make it more readable.
+        no_validate : bool
+            Do not validate the reply against the actor schema. This is
+            ignored if the actor was not started with knowledge of its own
+            schema.
         kwargs
             Keyword arguments that will used to update the message.
 
         """
 
+        def send_to_transport(transport, message):
+
+            if getattr(transport, 'multiline', False):
+                message_json = json.dumps(message, sort_keys=False, indent=4) + '\n'
+            else:
+                message_json = json.dumps(message, sort_keys=False) + '\n'
+            transport.write(message_json.encode())
+
         message = message or {}
         assert isinstance(message, dict), 'message must be a dictionary'
         message.update(kwargs)
 
+        if not no_validate and self.schema is not None:
+            result, err = self.schema.update_model(message)
+            if result is False:
+                message = {'error': f'Failed validating the reply: {err}'}
+
         commander_id = command.commander_id if command else None
         command_id = command.command_id if command else None
         transport = command.transport if command else None
@@ -361,129 +381,30 @@ class JSONActor(ClickParser, BaseActor):
         header = {'header': {'command_id': command_id,
                              'commander_id': commander_id,
                              'message_code': message_code,
-                             'sender': self.name}
-                  }
+                             'sender': self.name}}
 
         message_full.update(header)
         message_full.update({'data': message})
 
-        if beautify:
-            message_json = json.dumps(message_full, sort_keys=False, indent=4)
-        else:
-            message_json = json.dumps(message_full, sort_keys=False)
-
-        message_json += '\n'
-
         if broadcast or commander_id is None or transport is None:
             for transport in self.transports.values():
-                transport.write(message_json.encode())
+                send_to_transport(transport, message_full)
         else:
-            transport.write(message_json.encode())
+            send_to_transport(transport, message_full)
 
+        message_json = json.dumps(message_full, sort_keys=False) + '\n'
         log_reply(self.log, message_code, message_json)
 
 
-class TimerCommandList(list):
-    """A list of `.TimerCommand` objects that will be executed on a loop.
+@click.command(cls=CluCommand)
+@click.option('--on/--off', default=True, help='Turn multiline on/off.')
+async def multiline(command, on):
+    """Set multiline mode for the transport."""
 
-    Parameters
-    ----------
-    actor
-        The actor in which the commands are to be run.
-    resolution : float
-        In seconds, how frequently to check if any of the `.TimerCommand` must
-        be executed.
+    transport = getattr(command, 'transport', None)
+    if not transport:
+        return command.fail('The command has no transport.')
 
-    """
+    transport.multiline = on
 
-    def __init__(self, actor, resolution=0.5, loop=None):
-
-        self.resolution = resolution
-        self.actor = actor
-        self._task = None
-        self.loop = loop or asyncio.get_event_loop()
-
-        list.__init__(self, [])
-
-    def add_command(self, command_string, **kwargs):
-        """Adds a new `.TimerCommand`."""
-
-        self.append(TimerCommand(command_string, **kwargs))
-
-    async def poller(self):
-        """The polling loop."""
-
-        current_time = time.time()
-
-        while True:
-
-            for timer_command in self:
-                elapsed = current_time - timer_command.last_run
-                if elapsed > timer_command.delay:
-                    timer_command_task = self.loop.create_task(timer_command.run(self.actor))
-                    timer_command_task.add_done_callback(timer_command.done)
-
-            self._sleep_task = self.loop.create_task(asyncio.sleep(self.resolution))
-
-            await self._sleep_task
-            current_time += self.resolution
-
-    def start(self):
-        """Starts the loop."""
-
-        if self.running:
-            raise RuntimeError('poller is already running.')
-
-        self._task = self.loop.create_task(self.poller())
-
-        return self
-
-    async def stop(self):
-        """Cancel the poller."""
-
-        if not self.running:
-            return
-
-        self._task.cancel()
-
-        with suppress(asyncio.CancelledError):
-            await self._task
-
-    @property
-    def running(self):
-        """Returns `True` if the poller is running."""
-
-        if self._task and not self._task.cancelled():
-            return True
-
-        return False
-
-
-class TimerCommand(object):
-    """A command to be executed on a loop.
-
-    Parameters
-    ----------
-    command_string : str
-        The command string to run.
-    delay : float
-        How many seconds to wait between repeated calls.
-
-    """
-
-    def __init__(self, command_string, delay=1):
-
-        self.command_string = command_string
-        self.delay = delay
-
-        self.last_run = 0.0
-
-    async def run(self, actor):
-        """Run the command."""
-
-        await Command(self.command_string, actor=actor).parse()
-
-    def done(self, task):
-        """Marks the execution of a command."""
-
-        self.last_run = time.time()
+    return command.finish('Multiline mode is {}'.format('on' if on else 'off'))
