@@ -15,6 +15,7 @@ import time
 
 from sdsstools import get_logger, read_yaml_file
 
+from .model import Model
 from .tools import REPLY
 
 
@@ -83,13 +84,13 @@ class BaseClient(metaclass=abc.ABCMeta):
 
         pass
 
-    async def shutdown(self):
+    async def stop(self):
         """Shuts down all the remaining tasks."""
 
         self.log.info('cancelling all pending tasks and shutting down.')
 
-        tasks = [task for task in asyncio.Task.all_tasks(loop=self.loop)
-                 if task is not asyncio.tasks.Task.current_task(loop=self.loop)]
+        tasks = [task for task in asyncio.all_tasks(loop=self.loop)
+                 if task is not asyncio.current_task(loop=self.loop)]
         list(map(lambda task: task.cancel(), tasks))
 
         await asyncio.gather(*tasks, return_exceptions=True)
@@ -108,6 +109,8 @@ class BaseClient(metaclass=abc.ABCMeta):
 
         if 'actor' in config:
             config = config['actor']
+        elif 'client' in config:
+            config = config['client']
 
         return config
 
@@ -119,14 +122,16 @@ class BaseClient(metaclass=abc.ABCMeta):
         ----------
         config : dict or str
             A configuration dictionary or the path to a YAML configuration
-            file that must contain a section ``'actor'`` (if the section is
-            not present, the whole file is assumed to be the actor
-            configuration).
+            file. If the file contains a section called ``'actor'`` or
+            ``'client'``, that section will be used instead of the whole
+            file.
 
         """
 
         orig_config_dict = cls._parse_config(config)
+
         config_dict = orig_config_dict.copy()
+        config_dict.update(kwargs)
 
         # Decide what to do with the rest of the keyword arguments:
         args_inspect = inspect.getfullargspec(cls)
@@ -138,32 +143,33 @@ class BaseClient(metaclass=abc.ABCMeta):
         else:
             # Check the kw arguments in the subclass and pass only
             # values from config_dict that match them.
-            kw_args = args_inspect.kwonlyargs
-            if len(args_inspect.defaults) > 0:
-                args_invert = args_inspect.args[::-1]
-                kw_args += args_invert[:len(args_inspect.defaults)]
-            for kw in kwargs:
-                if kw in kw_args:
-                    config_dict[kw] = kwargs[kw]
+            class_kwargs = args_inspect.args
+            class_kwargs.remove('self')
+
+            # Remove keys that are not in the signature.
+            config_dict = {key: value for key, value in config_dict.items()
+                           if key in class_kwargs}
 
         # We also pass *args in case the actor has been subclassed
         # and the subclass' __init__ accepts different arguments.
-        new_actor = cls(*args, **config_dict)
+        new_client = cls(*args, **config_dict)
 
         # Store original config. This may not be complete since from_config
         # may have been super'd from somewhere else.
-        new_actor._config = orig_config_dict
-        new_actor._config.update(kwargs)
+        new_client._config = orig_config_dict
+        new_client._config.update(kwargs)
 
-        return new_actor
+        return new_client
 
     def setup_logger(self, log, log_dir, verbose=False):
         """Starts the file logger."""
 
         if not log:
-            log = get_logger('actor:' + self.name)
+            log = get_logger('clu:' + self.name)
 
-        if log_dir:
+        log.setLevel(REPLY)
+
+        if log is not False and log_dir:
 
             log_dir = pathlib.Path(log_dir).expanduser()
 
@@ -189,8 +195,8 @@ class BaseClient(metaclass=abc.ABCMeta):
 
         return log
 
-    def send_command(self):
-        """Sends a command to an actor."""
+    def send_command(self):  # pragma: no cover
+        """Sends a command to an actor and returns a `.Command` instance."""
 
         raise NotImplementedError('Sending commands is not implemented '
                                   'for this client.')
@@ -205,11 +211,25 @@ class BaseActor(BaseClient):
 
     """
 
-    @abc.abstractmethod
-    async def start(self):
-        """Starts the server. Must be overridden by the subclasses."""
+    schema = None
 
-        pass
+    def __init__(self, *args, schema=None, **kwargs):
+
+        super().__init__(*args, **kwargs)
+
+        self.validate_schema(schema)
+
+    def validate_schema(self, schema):
+        """Loads and validates the actor schema."""
+
+        schema = schema or self.schema
+
+        if schema is None:
+            return None
+
+        self.schema = Model(self.name, schema, is_file=True, log=self.log)
+
+        return self.schema
 
     @abc.abstractmethod
     def new_command(self):
