@@ -10,9 +10,8 @@ import warnings
 
 import clu
 
-from ..actor import TimerCommandList
 from ..base import BaseActor
-from ..command import Command, parse_legacy_command
+from ..command import Command, TimedCommandList, parse_legacy_command
 from ..parser import ClickParser
 from ..protocol import TCPStreamServer
 from ..tools import log_reply
@@ -45,7 +44,7 @@ class BaseLegacyActor(BaseActor):
         The host on which Tron is running.
     tron_port : int
         The port on which Tron is running.
-    model_names : list
+    models : list
         A list of strings with the actors whose models will be tracked.
     version : str
         The version of the actor.
@@ -57,6 +56,11 @@ class BaseLegacyActor(BaseActor):
     log : ~logging.Logger
         A `~logging.Logger` instance to be used for logging instead of creating
         a new one.
+    schema : str
+        The path to the datamodel schema for the actor, in JSON Schema format.
+        If the schema is provided all replies will be validated against it.
+        An invalid reply will fail and not be emitted. The schema can also be
+        set when subclassing by setting the class ``schema`` attribute.
 
     """
 
@@ -64,11 +68,12 @@ class BaseLegacyActor(BaseActor):
     port = None
 
     def __init__(self, name, host, port, tron_host=None, tron_port=None,
-                 model_names=None, version=None, loop=None, log_dir=None,
-                 log=None, verbose=False):
+                 models=None, version=None, loop=None, log_dir=None,
+                 log=None, verbose=False, schema=None):
 
         super().__init__(name, version=version, loop=loop,
-                         log_dir=log_dir, log=log, verbose=verbose)
+                         log_dir=log_dir, log=log, verbose=verbose,
+                         schema=schema)
 
         #: Mapping of user_id to transport
         self.transports = dict()
@@ -83,16 +88,15 @@ class BaseLegacyActor(BaseActor):
 
         if tron_host and tron_port:
             #: TronConnection: The client connection to Tron.
-            self.tron = TronConnection(tron_host, tron_port, self,
-                                       model_names=model_names, log=self.log)
+            self.tron = TronConnection(host=tron_host, port=tron_port,
+                                       models=models, log=self.log)
         else:
             self.tron = None
 
-        if self.tron:
-            #: dict: Actor models.
-            self.models = self.tron.models
+        #: dict: Actor models.
+        self.models = self.tron.models if self.tron else None
 
-        self.timer_commands = TimerCommandList(self)
+        self.timed_commands = TimedCommandList(self)
 
     def __repr__(self):
 
@@ -117,7 +121,7 @@ class BaseLegacyActor(BaseActor):
             warnings.warn(f'connection to tron was refused: {ee}. '
                           'Some functionality will be limited.', clu.CluWarning)
 
-        self.timer_commands.start()
+        self.timed_commands.start()
 
         return self
 
@@ -127,7 +131,7 @@ class BaseLegacyActor(BaseActor):
         if self._server.is_serving():
             self._server.stop()
 
-        await self.timer_commands.stop()
+        await self.timed_commands.stop()
 
         if self.tron:
             self.tron.stop()
@@ -271,12 +275,18 @@ class BaseLegacyActor(BaseActor):
         """
 
         if self.tron:
-            self.tron.send_command(target, command_string, mid=command_id)
+            command = self.tron.send_command(target, command_string,
+                                             commander=f'{self.name}.{self.name}',
+                                             mid=command_id)
+            command.actor = self
+            return command
+
         else:
             raise clu.CluError('cannot connect to tron.')
 
     def write(self, message_code='i', message=None, command=None, user_id=None,
-              command_id=None, concatenate=True, broadcast=False, **kwargs):
+              command_id=None, concatenate=True, broadcast=False,
+              no_validate=False, **kwargs):
         """Writes a message to user(s).
 
         Parameters
@@ -301,6 +311,10 @@ class BaseLegacyActor(BaseActor):
             each keyword will be output as a different message.
         broadcast : bool
             Whether to broadcast the reply. Equivalent to ``user_id=0``.
+        no_validate : bool
+            Do not validate the reply against the actor schema. This is
+            ignored if the actor was not started with knowledge of its own
+            schema.
         kwargs
             Keyword arguments that will be added to the message. If a keyword
             is both in ``message`` and in ``kwargs``, the value in ``kwargs``
@@ -327,6 +341,12 @@ class BaseLegacyActor(BaseActor):
         elif not isinstance(message, dict):
             raise TypeError('invalid message type ' + str(type(message)))
 
+        if not no_validate and hasattr(self, 'schema') and self.schema is not None:
+            result, err = self.schema.update_model(message)
+            if result is False:
+                message = {'error': f'Failed validating the reply: {err}'}
+                message_code = 'e'
+
         message.update(kwargs)
 
         lines = []
