@@ -9,6 +9,7 @@
 import asyncio
 import functools
 import inspect
+import json
 import re
 
 import click
@@ -17,8 +18,7 @@ from click.decorators import group, pass_obj
 from . import actor
 
 
-__all__ = ['CluCommand', 'CluGroup', 'command_parser',
-           'ClickParser', 'timeout']
+__all__ = ['CluCommand', 'CluGroup', 'command_parser', 'ClickParser', 'timeout']
 
 
 def coroutine(fn):
@@ -37,16 +37,26 @@ def coroutine(fn):
 class CluCommand(click.Command):
     """Override :py:class:`click.Command` to pass the actor and command."""
 
+    def __init__(self, *args, context_settings=None, **kwargs):
+
+        # Unless told otherwise, set ignore_unknown_options=True to prevent
+        # negative numbers to be considered as options. See #40.
+        if (context_settings is None or
+                'ignore_unknown_options' not in context_settings):
+            context_settings = context_settings or {}
+            context_settings['ignore_unknown_options'] = True
+
+        super().__init__(*args, context_settings=context_settings, **kwargs)
+
     def done_callback(self, task, exception_handler=None):
         """Checks if the command task has been successfully done."""
 
-        log = getattr(task, 'log', None)
         ctx = task.ctx
 
         command = ctx.obj['parser_args'][0]
 
         if exception_handler and task.exception():
-            exception_handler(command, task.exception(), log=log)
+            exception_handler(command, task.exception())
 
     async def _schedule_callback(self, ctx, timeout=None):
         """Schedules the callback as a task with a timeout."""
@@ -172,10 +182,17 @@ def timeout(seconds):
         # we add the timeout directly to the callback function.
         f.timeout = seconds
 
+        async def helper(f, *args, **kwargs):
+            if asyncio.iscoroutinefunction(f):
+                return await f(*args, **kwargs)
+            else:
+                return f(*args, **kwargs)
+
         @functools.wraps(f)
-        def new_func(*args, **kwargs):
-            return f(*args, **kwargs)
-        return functools.update_wrapper(new_func, f)
+        async def wrapper(*args, **kwargs):
+            return await helper(f, *args, **kwargs)
+
+        return functools.update_wrapper(wrapper, f)
 
     return decorator
 
@@ -208,6 +225,29 @@ def ping(*args):
     return
 
 
+@command_parser.command()
+def version(*args):
+    """Reports the version."""
+
+    command = args[0]
+    command.set_status(command.status.DONE,
+                       version=command.actor.version)
+
+    return
+
+
+@command_parser.command(cls=CluCommand, name='get_schema')
+def get_schema(*args):
+    """Returns the schema of the actor as a JSON schema."""
+
+    command = args[0]
+
+    if command.actor.schema is None:
+        return command.fail(text='The actor does not know its own schema.')
+
+    return command.finish(schema=json.dumps(command.actor.schema.schema))
+
+
 @command_parser.command(name='help')
 @click.argument('PARSER-COMMAND', type=str, required=False)
 @click.pass_context
@@ -272,12 +312,6 @@ class ClickParser:
     def parse_command(self, command):
         """Parses an user command using the Click internals."""
 
-        # Makes sure we have the ping and help commands available
-        if 'help' not in self.parser.commands:
-            self.parser.add_command(help_)
-        if 'ping' not in self.parser.commands:
-            self.parser.add_command(ping)
-
         # This will pass the command as the first argument for each command.
         # If self.parser_args is defined, those arguments will be passed next.
         parser_args = [command]
@@ -327,8 +361,7 @@ class ClickParser:
 
         return command
 
-    @staticmethod
-    def _handle_command_exception(command, exception, log=None):
+    def _handle_command_exception(self, command, exception):
         """Handles an exception during parsing or execution of a command."""
 
         try:
@@ -358,21 +391,12 @@ class ClickParser:
             if not command.status.is_done:
                 command.fail(text=msg)
             else:
-                command.write(text=msg)
-
-        except click.exceptions.Exit:
-
-            # This happens when using --help, although it should be handled
-            # in parse_command.
-            if command.status.is_done:
-                command.write(text='Use help [CMD]')
-            else:
-                command.fail(text='Use help [CMD]')
+                command.write('e', text=msg)
 
         except click.exceptions.Abort:
 
             if not command.status.is_done:
-                command.fail(text='Command was aborted.')
+                command.fail(text=f'Command {command.body!r} was aborted.')
 
         except Exception:
 
@@ -385,6 +409,6 @@ class ClickParser:
             else:
                 command.fail(text=msg)
 
-            log = log or getattr(command.ctx, 'log', None)
+            log = self.log or command.ctx.obj.get('log', None)
             if log:
                 log.exception(f'Command {command.body!r} failed with error:')
