@@ -8,16 +8,12 @@
 
 import asyncio
 
+import aio_pika as apika
+import aiormq
+
 from .exceptions import CluError
 
 
-try:
-    import aio_pika as apika
-    import aiormq
-except ImportError:
-    apika = None
-
-
 __all__ = ['TCPProtocol', 'PeriodicTCPServer',
            'TCPStreamServer', 'TCPStreamPeriodicServer',
            'TCPStreamClient', 'open_connection',
@@ -352,7 +348,8 @@ class TCPStreamPeriodicServer(TCPStreamServer):
     port : int
         The server port.
     period_callback
-        Callback to run every iteration.
+        Callback to run every iteration. It is called for each transport
+        that is connected to the server and receives the transport object.
     sleep_time : float
         The delay between two calls to ``periodic_callback``.
     kwargs : dict
@@ -379,6 +376,11 @@ class TCPStreamPeriodicServer(TCPStreamServer):
 
         return self._server
 
+    def stop(self):
+
+        self.periodic_task.cancel()
+        super().stop()
+
     @property
     def periodic_callback(self):
         """Returns the periodic callback."""
@@ -409,18 +411,22 @@ class TopicListener(object):
     ----------
     user : str
         The user to connect to the RabbitMQ broker.
+    password : str
+        The password for the user.
     host : str
-        The host where the RabbitMQ message broker lives.
+        The host where the RabbitMQ message broker runs.
+    port : int
+        The port on which the RabbitMQ message broker is running.
 
     """
 
-    def __init__(self, user=None, host=None):
-
-        if not apika:
-            raise ImportError('cannot use TopicListener without aoi_pika.')
+    def __init__(self, user='guest', password='guest',
+                 host='localhost', port=5672):
 
         self.user = user
+        self.password = password
         self.host = host
+        self.port = port
 
         self.connection = None
         self.channel = None
@@ -449,10 +455,9 @@ class TopicListener(object):
 
         if not channel:
 
-            assert self.user and self.host, 'either user or host are not set.'
-
             self.connection = await apika.connect_robust(user=self.user,
                                                          host=self.host,
+                                                         port=self.port,
                                                          loop=self.loop)
 
             self.channel = await self.connection.channel()
@@ -493,7 +498,7 @@ class TopicListener(object):
         elif isinstance(bindings, (list, tuple)):
             bindings = list(bindings)
         else:
-            raise TypeError('invalid type for bindings {bindings!r}.')
+            raise TypeError(f'invalid type for bindings {bindings!r}.')
 
         try:
             queue = await self.channel.declare_queue(queue_name, exclusive=True)
@@ -505,9 +510,19 @@ class TopicListener(object):
         for binding in bindings:
             await queue.bind(self.exchange, routing_key=binding)
 
+        if callback:
+            queue.consumer_tag = await queue.consume(callback)
+
         self.queues.append(queue)
 
-        if callback:
-            queue.consumer_task = self.loop.create_task(queue.consume(callback))
-
         return queue
+
+    async def stop(self):
+        """Cancels queues and closes the connection."""
+
+        for queue in self.queues:
+
+            if hasattr(queue, 'consumer_tag') and queue.consumer_tag is not None:
+                await queue.cancel(queue.consumer_tag)
+
+        await self.connection.close()
