@@ -41,7 +41,7 @@ COMMAND_STATUS_TO_CODE = {
     'FAILED': 'f',
     'TIMEDOUT': 'f',
     'READY': 'i',
-    'RUNNING': 'i',
+    'RUNNING': '>',
     'CANCELLING': 'w',
     'FAILING': 'w',
     'DEBUG': 'd',
@@ -112,15 +112,21 @@ class CommandStatus(Maskbit):
         return self in self.FAILING_STATES
 
     @staticmethod
-    def get_inverse_dict():
-        """Gets a reversed dictionary of code to status.
+    def code_to_status(code, default=None):
+        """Returns the status associated with a code.
 
-        Note that the inverse dictionary is not unique and you can get
-        different statuses associated with the same code.
+        If the code doesn't have an associated status, returns ``default``.
+        ``default`` defaults to `.CommandStatus.RUNNING`.
 
         """
 
-        return dict((status.code, status) for status in CommandStatus if status.code)
+        statuses = {':': CommandStatus.DONE,
+                    'f': CommandStatus.FAILED,
+                    'e': CommandStatus.FAILED,
+                    '!': CommandStatus.FAILED,
+                    '>': CommandStatus.RUNNING}
+
+        return statuses.get(code, default or CommandStatus.RUNNING)
 
 
 class StatusMixIn(object):
@@ -182,7 +188,7 @@ class StatusMixIn(object):
         return self._status
 
     @status.setter
-    def status(self, value):
+    def status(self, value):  # pragma: no cover
         """Sets the status."""
 
         if value != self._status:
@@ -527,10 +533,9 @@ class ActorHandler(logging.Handler):
         message_lines = [f'{warning_text} ({warning_category})']
 
         try:
-            warning_class = eval(warning_category)
             if self.filter_warnings:
                 for warning_filter in self.filter_warnings:
-                    if isinstance(warning_class, warning_filter):
+                    if warning_category == warning_filter.__name__:
                         return message_lines
             return []
         except NameError:
